"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
function openEditor(file, opts = {}) {
    return new Promise((resolve, reject) => {
        const editor = getEditor(opts.editor);
        const args = editor.split(/\s+/);
        const bin = args.shift();
        if (!bin) {
            reject(new Error('Editor binary not found'));
            return;
        }
        const ps = child_process_1.spawn(bin, [...args, file], { stdio: 'inherit' });
        ps.on('exit', () => {
            resolve();
        });
        ps.on('error', (err) => {
            reject(err);
        });
    });
}
exports.openEditor = openEditor;
function getEditor(editor) {
    return (editor || process.env.VISUAL || process.env.EDITOR || getDefaultEditor());
}
function getDefaultEditor() {
    return /^win/.test(process.platform) ? 'notepad' : 'vim';
}
exports.default = openEditor;
